# /psets/2-notes/new.tck
#
# Copyright (C) 1998-2007 various parties
#
# This program is free software. You can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# Edited by <frank.bergmann@project-open-com>

# ---------------------------------------------------------------
# Page Contract
# ---------------------------------------------------------------

ad_page_contract {
    View, Edit and Create "notes".
    All of these three operations are handled by this single page.
    @author frank.bergmann@project-open.com

    @param note_id The ID of the note object. If the note_id is not
	   specified then the ad_form will start to create a new objects.
    @param object_id Specifies that the note should be assoicated
	   with another objects. Optional.
    @param return_url Specifies where to go after a successful 
	   Edit or Create operations.
    @param form_mode specifies how an existing object should be shown.
	   "display" triggers read-only mode, while "edit" allows the
	   used to modify the object.
} {
    note_id:integer,optional
    {object_id:integer "" }
    {return_url "index"}
    {form_mode "edit"}
}

# ---------------------------------------------------------------
# Defaults & Security
# ---------------------------------------------------------------

set user_id [ad_maybe_redirect_for_registration]
set page_title "View, Edit and Create Notes"
set context_bar [im_context_bar $page_title]


# "Options" for the note_type. The format is a list of lists
# with 1. the pretty name and 2. the value for the <select> tag.
set note_type_options {
    { Telephone tel }
    { "Cell Phone" cell }
    { Fax fax }
    { URL web }
    { Email email }
    { Address address }
    { Other other }
} 

# ---------------------------------------------------------------
# Defaults & Security
# ---------------------------------------------------------------

# Start the "Form". Please see the documentation of ad_form for details
# (go to /api-doc/ and enter "ad_form" in the first field at the top right).
# The -form part contains a list of field definitions.
#	- The first "note_id:key" specifies the "note_id" as the "key" for the form.
#	- The other two fields contain information on what they are and how they
#	  are to be rendered. Each field consists of:
#		- field_name:type
#		- A label to be used for the left-hand column
#		- Additional information on how to render the field.
# 
ad_form \
    -name notes-form \
    -mode $form_mode \
    -export "object_id return_url" \
    -form {
	note_id:key
	{
	    note_type:text(select) 
	    {label Type} 
	    {options $note_type_options} 
	}
	{
	    note:text(textarea) 
	    {label Note} 
	    {html {cols 40} {rows 8}} 
	}
    }


# Define the behaviour of the form for different operations:
#	- select_query: Define how the ad_form pulls out data from 
#	  the db when editing an existing object
#	- new_data: Defines how to create a new object
#	- edit_data: Defines how to update an existing object
#	- after_submit: Defines how the ad_form should proceed
#	  after "Submit" has been pressed.
#
ad_form -extend -name notes-form \
    -select_query {
	select	*
	from	notes
	where	note_id = :note_id
    } -new_data {
	db_exec_plsql create_note "
		SELECT note__new(
			:note_id,
			'note',
			now(),
			:user_id,
			'[ad_conn peeraddr]',
			null,
			:note,
			:object_id,
			:note_type
		)
        "
    } -edit_data {
	db_dml edit_note "
		update notes set 
			note = :note,
			note_type = :note_type
		where note_id = :note_id
	"
    } -after_submit {
	ad_returnredirect $return_url
	ad_script_abort
    }


