
<html>
<head>
<title>Control Structure</title>
<LINK REL=STYLESHEET HREF="tcl_manual.css" TYPE="text/css">
</head>
<body bgcolor=white text=black>

<h2>Control Structure</h2>

part of <a href="index.adp">Tcl for Web Nerds</a> 
by 
<a href="http://www-swiss.ai.mit.edu/~hal/hal.html">Hal Abelson</a>,
<a href="http://philip.greenspun.com/">Philip Greenspun</a>,
and
<a href="http://sandon.org/l/">Lydia Sandon</a>


<hr>


Control structures let you say "run this fragment of code if X is
true" or "do this a few times" or "do this until something is no
longer true".  The available control structures in Tcl may be grouped
into the following categories:

<ul>

<li>conditional
<li>looping (iteration)
<li>error-handling
<li>miscellaneous (non-local exit)
</ul>

<H3>The Fundamental Conditional Command: <code>if</code></H3>

The most basic Tcl control structure is the <code>if</code> command:

<blockquote>
<code><pre>
if boolean ?then? body1 ?else? ?body2?
</pre></code>
</blockquote>


Note that the words "then" and "else" are optional, as is the entire
else clause.  The most basic <code>if</code> statement looks like
this:

<blockquote>
<code><pre>
if {<i>condition</i>} {
   <i>body</i>
}
</pre></code>
</blockquote>


In the ArsDigita Community System, we always leave out the "then", but
if we include an <code>else</code> or <code>elseif</code> clause, we
put in those optional words.  Consistency is the hobgoblin of little
minds...

<blockquote>
<code><pre>
if {<i>condition</i>} {
   <i>body</i>
} elseif {<i>other_condition</i>} {
   <i>alternate_body</i>
} else {
   <i>another_body</i>
}
</pre></code>
</blockquote>


Note how the curly braces and keywords are artfully positioned so that
the entire <code>if</code> statement is on one line as far as the
interpreter is concerned, i.e., all the newlines are grouped within
curly braces.  An easy way to break your program is to rewrite the
above statement as follows:

<blockquote>
<code><pre>
if {<i>condition</i>} {
   <i>body</i>
} 
elseif {<i>other_condition</i>} {
   <i>alternate_body</i>
} else {
   <i>another_body</i>
}
</pre></code>
</blockquote>


The Tcl interpreter will think that the <code>if</code> statement has
ended after the first body and will next try to evaluate "elseif" as a
procedure.

<p class="stb">

Let's look at an example from <a
href="http://software.arsdigita.com/www/register/user-login.tcl">http://software.arsdigita.com/www/register/user-login.tcl</a>.
At this point in the ArsDigita Community System flow, a user has
already typed his or her email address.  

<blockquote>
<code><pre>
# Get the user ID
set selection [ns_db 0or1row $db "select user_id, user_state,
converted_p 
from users
where upper(email)=upper('$QQemail')"]

if {$selection == ""} {
    # Oracle didn't find a row; this email addres is not in the database
    # redirect this person to the new user registration page
    ns_returnredirect "user-new.tcl?[export_url_vars return_url email]"
    return
} 
</pre></code>
</blockquote>


The same page provides an example both of nested <code>if</code> and
<code>if then else</code>:

<blockquote>
<code><pre>
if [ad_parameter AllowPersistentLoginP "" 1] {
    # publisher has elected to provide an option to issue
    # a persistent cookie with user_id and crypted password
    if [ad_parameter PersistentLoginDefaultP "" 1] {
        # persistent cookie shoudl be the default
	set checked_option "CHECKED" 
    } else {
	set checked_option "" 
    }
    ns_write "&lt;input type=checkbox name=persistent_cookie_p value=t $checked_option&gt; 
Remember this address and password?  
(&lt;a href=\"explain-persistent-cookies.adp\"&gt;help&lt;/a&gt;)"
}
</pre></code>
</blockquote>


Notice that the conventional programming style in Tcl is to call
<code>if</code> for effect rather than value.  It would work just as
well to write the inner <code>if</code> in a more Lisp-y style:

<blockquote>
<code><pre>
set checked_option [if [ad_parameter ...] {
    subst "CHECKED" 
} else {
    subst ""
}]
</pre></code>
</blockquote>


This works because <code>if</code> returns the value of the last
expression evaluated.  However, being correct and being comprehensible
to the community of Tcl programmers are different things.  It is best
to write code adhering to indentation and other stylistic
conventions.  You don't want to be the only person in the world
capable of maintaining a service that has to be up 24x7.

<H3>Another Conditional Command: <code>switch</code></H3>

The <code>switch</code> dispatches on the value of its first argument:
particular variable as follows:

<blockquote>
<code><pre>
switch <i>flags</i> <i>value</i> {
   <i>pattern1</i> <i>body1</i>
   <i>pattern2</i> <i>body2</i>
   ...
}
</pre></code>
</blockquote>


If <a
href="http://software.arsdigita.com/www/register/user-login.tcl">http://software.arsdigita.com/www/register/user-login.tcl</a>
finds a user in the database, it uses a <code>switch</code> on the
user's state to decide what to do next:

<blockquote>
<code><pre>
switch $user_state {
    "authorized" { # just move on } 
    "banned" { 
	ns_returnredirect "banned-user.tcl?user_id=$user_id" 
	return
    }
    "deleted" {  
	ns_returnredirect "deleted-user.tcl?user_id=$user_id" 
	return
    }
    "need_email_verification_and_admin_approv" {
	ns_returnredirect "awaiting-email-verification.tcl?user_id=$user_id"
	return
    }
    "need_admin_approv" { 
	ns_returnredirect "awaiting-approval.tcl?user_id=$user_id"
	return
    }
    "need_email_verification" {
	ns_returnredirect "awaiting-email-verification.tcl?user_id=$user_id"
	return
    }
    "rejected" {
	ns_returnredirect "awaiting-approval.tcl?user_id=$user_id"
	return
    }
    default {
	ns_log Warning "Problem with registration state machine on user-login.tcl"
	ad_return_error "Problem with login" "There was a problem authenticating the account: $user_id. Most likely, the database contains users with no user_state."
	return
    }
}
</pre></code>
</blockquote>


In this case, we're using the standard <code>switch</code> behavior of
matching strings exactly.  We're also provide a "default" keyword at
the end that indicates some code to run if nothing else matched.

<p>

It is possible to use more sophisticated patterns in
<code>switch</code>.  Here's a fragment that sends different email
depending on the pattern of the address:

<blockquote>
<code><pre>
switch -glob $email {
   {*mit.edu} { ns_sendmail $email $from $subject $body }
   {*cmu.edu} { ns_sendmail $email $from $subject "$body\n\nP.S. Consider applying to MIT.  Boston is much nicer than Pittsburgh"}
   {*harvard.edu} { ns_sendmail $email $from $subject "$body\n\nP.S. Please ask your parents to invest in our tech startup."}
}
</pre></code>
</blockquote>


The third behavior for <code>switch</code> is invoked using the
"-regexp" flag.  See the <a href="pattern-matching.adp">pattern
matching chapter</a> for more on how these patterns work.

<br>
<br>

More: <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/switch.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/switch.n.html</a> 


<H3>Looping commands <code>while</code>,
<code>foreach</code>, and <code>for</code></H3>

The <code>while</code> command in Tcl operates as follows:

<blockquote>
<code><pre>
while { <i>conditional_statement</i> } {
   <i>loop_body_statements</i>
}
</pre></code>
</blockquote>


The conditional statement is evaluated; if it
is true, the loop body statement is executed, and then the conditional
statement is reevaluated and the process repeats.  If the conditional
statement is ever false, the interpreter does not execute the loop
body statements, and continues to the next line after the conditional.

<p>


Here is a <code>while</code> statement used to display the last name,
first name of each MIT nerd using a Web service.  The conditional
is the result of calling AOLserver's
<code>ns_db getrow</code> API procedure.  This procedure returns 1 if
it can fetch the next row from the SQL cursor, 0 if there aren't any
more rows to fetch.

<blockquote>
<code><pre>
set selection [ns_db select $db "select first_names, last_name 
from users
where lower(email) like '%mit.edu'"]

while { [ns_db getrow $db $selection] } { 
    # set local variables to the SQL column names
    set_variables_after_query
    ns_write "&lt;LI&gt;$last_name, $first_names"
}
</pre></code>
</blockquote>


More:  see <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/while.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/while.n.html</a>

<p class="mtb">

The Tcl <code>foreach</code> command loops through the elements of a
list, setting a loop variable to each element in term:

<blockquote>
<code><pre>
foreach <i>variable_name list</i> {
   <i>body</i>
}
</pre></code>
</blockquote>


Here's an example from <a
href="http://software.arsdigita.com/www/monitor.tcl">http://software.arsdigita.com/www/monitor.tcl</a>,
a page that displays current server activity:

<blockquote>
<code><pre>
# ask AOLserver to return a list of lists, one for each current connection
set connections [ns_server active]

foreach connection $connections {
    # separate the sublist elements with "<td>" tags
    ns_write $conn "<tr><td>[join $connection "<td>"]"
}
</pre></code>
</blockquote>


The program <a
href="http://sofware.arsdigita.com/www/admin/static/link-check.tcl">http://sofware.arsdigita.com/www/admin/static/link-check.tcl</a>
checks every HTML file in an ArsDigita Community System for dead
links.  Here's a helper procedure that works on one file:

<blockquote>
<code><pre>
proc check_file {f} {
    # alert the administrator that we're working on this file
    ns_write "&lt;li&gt;$f\n&lt;ul&gt;\n"
    # read the contents into $content
    set stream [open $f]
    set content [read $stream]
    close $stream
    # loop through each reference, relying on API call ns_hrefs
    # to parse the HTML and tell us where this file points
    <font color=red>foreach url [ns_hrefs $content] {</font>
	# do all the hard work
        ...
    }
    ns_write "&lt;/ul&gt;\n"
}
</pre></code>
</blockquote>


Notice how easy this procedure was to write thanks to the AOLserver
developers thoughtfully providing us with <code>ns_hrefs</code>, which
takes an HTML string and returns a list of every HREF target.

<br>
<br>

More:  see <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/foreach.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/foreach.n.html</a>

<p class="mtb">

The last looping command, <code>for</code>, is good for traditional
"for i from 1 to 10" kind of iteration.  Here's the syntax:

<blockquote>
<code><pre>
for <i>start test next body</i>
</pre></code>
</blockquote>


We use this control structure in the winner picking admin page of the
ArsDigita Comunity System's contest module:
<a
href="http://software.arsdigita.com/www/admin/contest/pick-winners.tcl">http://software.arsdigita.com/www/admin/contest/pick-winners.tcl</a>.
The input to this page specifies a time period, a contest, and how
many winners are to be picked.  Here the result of executing the
<code>for</code> loop is a list of N elements, where N is the number
of desired winners:

<blockquote>
<code><pre>
for {set i 1} {$i <= $n_winners} {incr i} {
    # we'll have winner_numbers between 0 and $n_contestants - 1
    # because randomRange returns a random integer between 0 
    # and its argument
    lappend winner_numbers [randomRange $n_contestants]
}
</pre></code>
</blockquote>



<br>
<br>

More:  see <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/for.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/for.n.html</a>

<H3>Error-handling command: <code>catch</code></H3>

If a Tcl command throws an error in a CGI script or an AOLserver API
page, by default the user will be presented with an error page.  If
you don't want that to happen, fix your bugs!  Sometimes it isn't
possible to fix your bugs.  For example, the <code>ns_httpget</code>
API procedure fetches a Web page from the wider Internet.  Under
certain network-dependent conditions, it may throw an error.  If you
don't want your users to be exposed to that as an error, put in a
catch:

<blockquote>
<code><pre>
catch <i>script ?variable_name?</i>
</pre></code>
</blockquote>


<code>catch</code> returns 1 if <code>script</code> threw an error, 0
otherwise.  If you supply the second argument
(<code>variable_name</code>), <code>catch</code> will set that
variable to the result of executing <code>script</code>, whether or
not the script threw an error.

<P>

Our classic example always involves ns_httpget.  Here's one from
<a
href="http://www.webho.com/WealthClock">http://www.webho.com/WealthClock</a>:

<blockquote>
<code><pre>
# define a procedure that computes the entire page
proc wealth_ReturnWholePage {} {
    # do a couple of ns_httpgets and some arithmetic
    # to produce the user-visible HTML 
    ...
}

# this is the procedure registered to http://www.webho.com/WealthClock
proc wealth_Top {ignore} {
    if [catch {set moby_string [Memoize wealth_ReturnWholePage]} errmsg] {
	# something went wrong with our sources 
        ... return an apology message to the users
    } else {
	# we computed the result (or Memoize got it from the cache)
	ns_return 200 text/html $moby_string
    }
}
</pre></code>
</blockquote>


<p class="mtb">

Sending email is another time that a Web server has to go outside its
carefully controlled world and might experience an error.  Here is the
entire <a
href="http://software.arsdigita.com/tcl/ad-monitor.tcl">http://software.arsdigita.com/tcl/ad-monitor.tcl</a>,
which implements a central facility for other sections in the
ArsDigita Community System.  The idea is that programmers can put in
"email the administrator if broken" instructions on pages that won't
result in a nightmare for the administrator if the page is getting hit
every few seconds.

<blockquote>
<code><pre>
# the overall goal here is that the ad_host_administrator gets
# notified if something is horribly wrong, but not more than once
# every 15 minutes

# we store the last [ns_time] (seconds since 1970) notification time 
# in ad_host_administrator_last_notified

ns_share -init { set ad_host_administrator_last_notified 0 } ad_host_administrator_last_notified

proc ad_notify_host_administrator {subject body {log_p 0}} {
    ns_share ad_host_administrator_last_notified
    if $log_p {
	# usually the error will be in the error log anyway
	ns_log Notice "ad_notify_host_administrator: $subject\n\n$body\n\n"
    }
    if { [ns_time] > [expr $ad_host_administrator_last_notified + 900] } {
	# more than 15 minutes have elapsed since last note
	set ad_notify_host_administrator [ns_time]
	if <font color=red>[catch { ns_sendmail [ad_host_administrator] [ad_system_owner] $subject $body } errmsg]</font> {
	    ns_log Error "failed sending email note to [ad_host_administrator]"
	}
    }
}
</pre></code>
</blockquote>


<p class="mtb">

Make sure that you don't overuse catch.  The last thing that you want
is a page failing silently.  Genuine errors should always be brought
to a user's attention and ideally to the site administrator's.  Users
should not think that a server has done something on their behalf when
in fact the task was not accomplished.

<br>
<br>

More:  <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/catch.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/catch.n.html</a> 

<H3>Miscellaneous commands: <code>break</code>,
<code>continue</code>, <code>return</code>, and <code>error</code></H3>

When inside a looping command, it is sometimes desirable to get the
command to stop looping or to stop executing the current iteration but
to continue on the next one.  The <b><code>break</code></b> command is
used to permanently escape the loop; the <b><code>continue</code></b>
command is used to escape the current iteration of the loop but to
start again at the next iteration.  The syntax for each consists only
of the appropriate word written on a line by itself within a loop.  

<P>

We often use the <code>break</code> command when we want to limit the
number of rows to display from the database.  Here's an example from
the photo.net neighbor-to-neighbor system.  By default, we only want
to show a "reasonable" number of postings on one page:


<blockquote>
<code><pre>
set selection [ns_db select $db ... big SQL query ... ]

set list_items ""
# see what the publisher thinks is a reasonable number (default to 100)
set n_reasonable [ad_parameter NReasonablePostings neighbor 100]

# initialize a counter of the number of rows displayed so far
set counter 0
while {[ns_db getrow $db $selection]} {
    set_variables_after_query
    incr counter
    if { $counter &gt; $n_reasonable) } {
	# append ellipses 
	append list_items "&lt;p&gt;\n..."
	# flush the database cursor (tell Oracle that we don't need the
	# rest of the rows)
	ns_db flush $db 
	# break out of the loop
	<font color=red>break</font>
    }
    append list_items "&lt;li&gt;&lt;a href=\"view-one.tcl ..."

}
</pre></code>
</blockquote>


<br>
<br>

More:  <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/break.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/break.n.html</a>

<p class="mtb">

The <code>return</code> command has been shown before.  It
quits the proc it's in and returns the supplied value.  Remember that
any procedure lines after <code>return</code> aren't executed.  Too
many times we've seen code of the following form:

<blockquote>
<code><pre>
proc a_new_programmers_proc {} {
    set db [ns_db gethandle]
    # do a bunch of stuff with the database
    return $result
    # release the database handle
    ns_db releasehandle $db 
}
</pre></code>
</blockquote>


The most interesting thing that you can do with return is write
procedures that force their callers to return as well.  Here's an
example from <a
href="http://software.arsdigita.com/tcl/ad-security.tcl">http://software.arsdigita.com/tcl/ad-security.tcl</a>:

<blockquote>
<code><pre>
proc ad_maybe_redirect_for_registration {} {
    if { [ad_verify_and_get_user_id] != 0 } {
	# user is in fact logged in, return happiness
	return
    } else {
	ns_returnredirect "/register/index.tcl?return_url=[ns_urlencode [ns_conn url]$url_args]"
	# blow out of 2 levels
	return -code return
    }
}
</pre></code>
</blockquote>


A .tcl page can simply call this in-line 

<blockquote>
<code><pre>
ad_maybe_redirect_for_registration 

# the code below will never get executed if the user isn't registered
# ... update the database or whatever ...
</pre></code>
</blockquote>


<br>
<br>

More:  <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/return.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/return.n.html</a>

<P class="mtb">

The <code>error</code> command returns from a proc and and raises an
error that, if not caught by a <code>catch</code> statement, will
result in the user seeing a server error page.  The first argument to
<code>error</code> is displayed in the debugging backtrace:

<blockquote>
<code><pre>
proc divide {x y} {
   if {$y == 0} {
      error "Can't divide by zero."
   } else {
      return [expr $x / $y]
   }
}     
</pre></code>
</blockquote>


<br>
<br>

More:  <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/error.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/error.n.html</a>


<br>
<br>
Continue on to <A HREF="procedures.adp">procedures</A>.


<hr>
Return to <a href="index.adp">Table of Contents</a>
<br>
<br>
<address><a href="mailto:lsandon@alum.mit.edu">lsandon@alum.mit.edu</a></address>

<center><a href="/comments/add?page_id=4729">Add a comment</a> | <a href="/links/add?page_id=4729">Add a link</a></center>

</body>
</html>

