
<html>
<head>
<title>Introduction</title>
<LINK REL=STYLESHEET HREF="tcl_manual.css" TYPE="text/css">
</head>
<body bgcolor=white text=black>

<h2>Introduction</h2>

part of <a href="index.adp">Tcl for Web Nerds</a> 
by 
<a href="http://www-swiss.ai.mit.edu/~hal/hal.html">Hal Abelson</a>,
<a href="http://philip.greenspun.com/">Philip Greenspun</a>,
and
<a href="http://sandon.org/l/">Lydia Sandon</a>


<hr>


If you're an MIT student reading this tutorial, you have a special
advantage in learning Tcl, because you've read <a
href="http://www.amazon.com/exec/obidos/ASIN/0262011530/pgreenspun-20"><cite>Structure
and Interpretation of Computer Programs</cite></a> (Abelson and
Sussman 1996; MIT Press).  We're not going to try to redeliver the
knowledge in <cite>SICP</cite> packaged up in Tcl syntax.  However, we
are going to try to show how some of the big ideas from
<cite>SICP</cite> can be accomplished even in the comparatively
impoverished Tcl language.

<p class="stb">

One of the strengths of Lisp, the programming language used to
illustrate the principles in <cite>SICP</cite>, is that it has very
simple syntax:  

<blockquote>
<code><pre>(procedure-name arg1 arg2 arg3)</pre></code>
</blockquote>


e.g., 

<blockquote>
<code><pre>(set! ten-factorial (factorial 10))
(set! checking-account-balance (+ 25 34 86))
(print "If you're so smart, why aren't you rich like Bill Gates?")</pre></code>
</blockquote>


will set <code>ten-factorial</code> to the result of evaluating the
<code>factorial</code> procedure with an argument of <code>10</code>,
then compute a graduate student's checking account balance, then print
out some career advice to a computer science graduate student.

<p class="marginnote">

These examples are written in the Scheme dialect of Lisp, which is
distributed from
<a
href="http://www-swiss.ai.mit.edu/scheme-home.html">http://www-swiss.ai.mit.edu/scheme-home.html</a>. 

<p class="stb">
How would you accomplish the same goals in Tcl?  Tcl syntax is 

<blockquote>
<code><pre>procedure_name arg1 arg2 arg3
procedure_name arg1 [subprocedure subarg1 subarg2]</pre></code>
</blockquote>


This is like Lisp, except that procedures invoked at top level need
not be surrounded by delimeters.  Procedures invoked from within other
procedure calls must be surrounded by <code>[]</code> instead of
<code>()</code>:

<blockquote>
<code><pre>set ten_factorial [factorial 10]
set checking_account_balance [expr 25 + 34 + 86]
puts "If you're so smart, why aren't you rich like Bill Gates?"</pre></code>
</blockquote>


Tcl variables can't include dashes; you have to use underscore
instead.  Note that doing arithmetic with Tcl requires
escaping into a world of infix syntax with the <code>expr</code>
procedure.

<h3>Evaluation and quoting</h3>

Each line of Tcl is interpreted as a separate command:

<blockquote>
<code><pre>procedure_name arg1 arg2 arg3</pre></code>
</blockquote>


Arguments are evaluated in sequence.  The resulting values are then
passed to <code>procedure_name</code>, which is assumed to be a
system- or user-defined procedure.  Tcl assumes that you're mostly
dealing with strings and therefore stands some of the conventions of
standard programming languages on their heads.  For example, you might
think that <code>set foo bar</code> would result in Tcl complaining
about an undefined variable (<code>bar</code>).  But actually what
happens is that Tcl sets the variable <code>foo</code> to the
character string "bar":

<blockquote>
<code><pre>&gt; tclsh
% set foo bar
bar
% set foo 
bar
% </pre></code>
</blockquote>


<p>

The first command line illustrates that the <code>set</code> command
returns the new value that was set (in this case, the character string
"bar"), which is the result printed by the interpreter.  The second
command line uses the <code>set</code> command again, but this time
with only one argument.  This actually gets the value of the variable
<code>foo</code>.  Notice that you don't have to declare variables
before using them.

<p class="marginnote">By analogy with Scheme, you'd think that we
could just type the command line <code>$foo</code> and have Tcl return
and print the value.  This won't work in Tcl, however, which assumes
that every command line invokes a procedure.  This is why we need to
explicity use <code>set</code> or <code>puts</code>.

<p>

Does this mean that you never need to use string quotes when you've
got string literals in your program?  No.  In Tcl, the double quote is
a <em>grouping</em> mechanism.  If your string literal contains any
spaces, which would otherwise be interpreted as argument separators,
you need to group the tokens with double quotes:

<blockquote>
<code><pre>
% set the_truth Lisp is the world's best computer language
wrong # args: should be "set varName ?newValue?"
% set the_truth "Lisp is the world's best computer language"
Lisp is the world's best computer language
</pre></code>
</blockquote>


In the first command above, the Tcl interpreter saw that we were
attempting to call <code>set</code> with seven arguments.  In the
second command, we grouped all the words in our string literal with
the double quotes and therefore the Tcl interpreter saw only two
arguments to <code>set</code>.  Note a stylistic point here:
multi-word variable names are all-lowercase with underscores
separating the words.  This makes our Tcl code very compatible with
relational database management systems where underscore is a legal
character in a column name.


<p>In this example, we invoked the Unix command "tclsh" to start the Tcl
interpreter from the Unix shell.  Later on we'll see how to use Tcl in
other ways:
<ul>
<li>writing a program file and evaluating it
<li>embedding Tcl commands in Web pages to create dynamic pages
<li>extending the behavior of the Web server with Tcl programs
</ul>
For now, let's stick with typing interactively at the shell.  You can
keep evaluating Tcl commands at the <code>%</code> prompt until you
exit the Tcl shell by evaluating <code>exit</code>.

<p class="mtb">

To indicate a literal string that contains a space, you can wrap the string in
double quotes.  Quoting like this does not prevent the interpreter
from evaluating procedure calls and variables inside the strings:

<blockquote>
<code><pre>
% set checking_account_balance [expr 25 + 34 + 86]
145
% puts "your bank balance is $checking_account_balance dollars"
your bank balance is 145 dollars
% puts "ten times your balance is [expr 10 * $checking_account_balance] dollars"
ten times your balance is 1450 dollars
</pre></code>
</blockquote>


The interpreter looks for dollar signs and square brackets within
quoted strings.  This is known as <em>variable interpolation</em> What
if you need to include a dollar sign or a square bracket?  One
approach is to escape with backslash:


<blockquote>
<code><pre>
% puts "your bank balance is \$$checking_account_balance"
your bank balance is $145
% puts "your bank balance is \$$checking_account_balance \[pretty sad\]"
your bank balance is $145 [pretty sad]
</pre></code>
</blockquote>


<p>If we don't need Tcl to evaluate variables and procedure calls
inside a string, we can use braces for grouping rather than double
quotes:

<blockquote>
<code><pre>
% puts {your bank balance is $checking_account_balance dollars}
your bank balance is $checking_account_balance dollars
% puts {ten times your balance is [expr 10 * $checking_account_balance] dollars}
ten times your balance is [expr 10 * $checking_account_balance] dollars
</pre></code>
</blockquote>


Throughout the rest of this book you'll see hundreds of examples of
braces being used as a grouping character for Tcl code.  For example,
when defining a procedure or using control structure commands,
conditional code is grouped using braces.

<h4>Keep it all on one line!</h4>

The good news is that Tcl does not suffer from cancer of the
semicolon.  The bad news is that any Tcl procedure call or command
must be on one line from the interpreter's point of view.  Suppose
that you want to split up

<blockquote>
<code><pre>
% set a_very_long_variable_name "a very long value of some sort..."
</pre></code>
</blockquote>


If you want to have newlines within the double quotes, that's just
fine:

<blockquote>
<code><pre>
% set a_very_long_variable_name "a very long value of some sort... 
with a few embedded newlines
makes for rather bad poetry"
a very long value of some sort... 
with a few embedded newlines
makes for rather bad poetry
% 
</pre></code>
</blockquote>


It also works to do it with braces

<blockquote>
<code><pre>
% set a_very_long_variable_name {a very long value of some sort... 
with a few embedded newlines
makes for rather bad poetry}
a very long value of some sort... 
with a few embedded newlines
makes for rather bad poetry
% 
</pre></code>
</blockquote>


but if you were to try 

<blockquote>
<code><pre>
set a_very_long_variable_name 
    "a very long value of some sort... 
     with a few embedded newlines
     makes for rather bad poetry"
</pre></code>
</blockquote>


Tcl would interpret this as two separate commands, the first a call to
<code>set</code> to find the existing value of
<code>a_very_long_variable_name</code> and the second a call to the
procedure named "a very long value...":

<blockquote>
<code><pre>
can't read "a_very_long_variable_name": no such variable
invalid command name "a very long value of some sort... 
     with a few embedded newlines
     makes for rather bad poetry"
</pre></code>
</blockquote>


If you want to continue a Tcl command on a second line, it is possible
to use the backslash to escape the newline that would otherwise
terminate the command:

<blockquote>
<code><pre>
% set a_very_long_variable_name \
      "a very long value of some sort... 
       with a few embedded newlines
       makes for rather bad poetry"
a very long value of some sort... 
       with a few embedded newlines
       makes for rather bad poetry
% 
</pre></code>
</blockquote>


Note that this looks good as code but the end-result is probably not
what you'd want.  The second and third lines of our poem contain seven
spaces at the beginning of each line.  You probably want to do
something like this:

<blockquote>
<code><pre>
% set a_very_long_variable_name "a very long value of some sort... 
with a few embedded newlines
makes for rather bad poetry"
</pre></code>
</blockquote>


For completeness we note that semicolon will allow you to put multiple
Tcl commands on one line:

<blockquote>
<code><pre>
% set x 2; set y 3; set z [expr $x+$y]
5
</pre></code>
</blockquote>


We never use this facility of the language.

<h4>Case Sensitivity, Poisonous Unix Heritage, and Naming Conventions</h4>

The great case-sensitivity winter descended upon humankind in 1970
with the Unix operating system.  

<blockquote>
<code><pre>
% set MyAge 36
36
% set YearsToExpectedDeath [expr 80-$Myage]
can't read "Myage": no such variable
% 
</pre></code>
</blockquote>


Variables and procedure names in Tcl are case-sensitive.  We consider
it very bad programming style to depend on this, though.  For example,
you shouldn't simultaneously use the variables <code>Username</code>
and <code>username</code> and rely on the computer to keep them
separate; the computer will succeed but humans maintaining the program
in the future will fail.

So use lowercase all the time with underscores to separate words!

<h3>Procedures</h3>

One of the keys to making a large software system reliable and
maintainable is <em>procedural abstraction</em>.  The idea is to take
a complex operation and encapsulate it into a function that other
programmers can call without worrying about how it works.

<p>

Here's the factorial procedure in Tcl:

<blockquote>
<code><pre>
% #this is good old recursive factorial
% proc factorial {number} {
    if { $number == 0 } {
	return 1
    } else {                 
	return [expr $number * [factorial [expr $number - 1]]]
    }
}
% factorial 10
3628800
</pre></code>
</blockquote>


At first glance, you might think that you've had to learn some new
syntax here.  In fact, the Tcl procedure-creation procedure is called
like any other.  The three arguments to <code>proc</code> are
<i>procedure_name arglist body</i>.  The creation command is able to
extend over several lines not because the interpreter recognizes
something special about <code>proc</code> but because we've used
braces to group blocks of code.  Similarly the <code>if</code>
statement within the procedure uses braces to group its arguments so
that they are all on one line as far as the interpreter is concerned.

<p>

As the example illustrates, we can use the standard
base-case-plus-recursion programming style in Tcl.  Our factorial
procedure checks to see if the number is 0 (the base case).  If so, it
returns 1.  Otherwise, it computes factorial of the number minus 1 and
returns the result multiplied by the number.  The <code>#</code>
character signals a comment.

<p>We'll have more to say about procedures later on.  For now, examine
the following example:

<blockquote>
<code><pre>
% set checking_account_balance [expr 25 + 34 + 86]
145
% puts "\nYour checking balance is \$$checking_account_balance.
If you're so smart, why aren't you rich like Bill Gates?
He probably has \$[factorial $checking_account_balance] by now."


Your checking balance is $145.
If you're so smart, why aren't you rich like Bill Gates?
He probably has $0 by now.
</pre></code>
</blockquote>


There are a few things to observe here:
<ul>

<li>The "\n" at the beginning of the quoted string argument to
<code>puts</code> resulted in an extra newline in front of the output.

<li>The newline after <code>balance.</code> did not terminate the
<code>puts</code> command.  The string quotes group all three lines
together into a single argument.

<li>The <code>[factorial... ]</code> procedure call was evaluated but
resulted in an output of 0.  This isn't a bug in the evaluation of
quoted strings, but rather a limitation of the Tcl language itself:
<blockquote>
<code><pre>% factorial 145
0</pre></code>
</blockquote>


</ul>

One of the reasons that commercial programmers work so hard and
accomplish so little is that they use languages that can't even do
arithmetic!  Tcl's feeble brain is overwhelmed by the idea of an
integer larger than one machine word (32 bits in this case).  You'll
have to go back to Scheme if you want to find Bill G's predicted
worth:

<blockquote>
<code><pre>(define (factorial number)
  (if (= number 0)
      1 
      (* number (factorial (- number 1)))))
;Value: factorial

(factorial 145)
;Value: 804792605747199194484902925779806277109997439007500616344745281047115412373646521410850481879839649227439298230298915019813108221651663659572441609408556917739149315905992811411866635786075524601835815642793302504243200000000000000000000000000000000000
</pre></code>
</blockquote>


<h3>Some powerful things about Tcl</h3>

A language that isn't powerful by itself can become powerful if a
procedure can invoke the interpreter, i.e., if a program can write a
program and then ask to have it evaluated.  In Lisp, you do this by
calling <code>eval</code>.  In Tcl, you do this ... by calling
<code>eval</code>!  We'll see examples of this later.

<p>Tcl also supports <em>reflection</em>:
You can write a Tcl program that pokes around the run-time environment.
One basic mechanism is

<blockquote>
<code><pre>info exists foobar</pre></code>
</blockquote>


which tests whether the variable <code>foobar</code> is
defined.  The <code>info</code> command can also tell a running
program what local variables are defined (<code>info local</code>), what
procedures are defined (<code>info procs</code>), what arguments a
procedure takes (<code>info args <i>procedure_name</i></code>), and what
the source code for a procedure is (<code>info body
<i>procedure_name</i></code>).

<p>

One useful aspect of reflection extends the behavior of
<code>set</code> described above:
a Tcl procedure
can examine the calling stack and set variables in its
caller's environment.
For example, in
<a
href="http://photo.net/wtr/thebook/community.html">the ArsDigita
Community System</a> toolkit, there's a procedure called
<code>set_variables_after_query</code> that looks at the last SQL query
executed and sets local variables to the values retrieved from the
database:

<blockquote>
<code><pre>set selection [ns_db 1row $db "select first_names,
last_name 
from users
where user_id = 37"]

set_variables_after_query 

ns_write "The above comment was made by $first_names $last_name.\n"
</pre></code>
</blockquote>


Note that this example uses procedures defined by AOLserver, e.g.,
<code>ns_db</code> to talk to the database and <code>ns_write</code>
to send bytes back to the Web client.  The prefixing of procedure
names by a package name is a good convention that you should follow in
your own programming.  In this case, the "ns_" prefix alerts fellow
programmers to the fact that you are calling API calls for the
"NaviServer".  NaviServer?  That's what AOLserver was called before
being purchased by America Online.  The fact that the API prefixes
were not changed illustrates another good programming practice: don't
change things gratuitiously.

<H3>Tcl for Web Use</H3>

In most cases, rather than typing Tcl expressions at the interpreter
you'll be using Tcl to generate dynamic Web pages.  AOLserver provides
two mechanisms for doing this:

<ol>
<LI><CODE>.tcl</CODE> pages are Tcl programs 
that are executed by
the webserver (in this case AOLServer).  They typically generate
character strings that are sent to the client browser with <code>ns_write</code>.

<LI><CODE>.adp</CODE> pages are like ordinary 
HTML pages, but they contain escapes to Tcl commands that are
evaluated by the server.
</ol>

<P>

Which option you choose depends largely on whether static HTML or Tcl
constitutes the majority of the page you are creating.  If you're
printing fixed character strings with Tcl, you'll need to
"quote" any embedded quotes by preceding them with backslash.  E.g.,
to print to a Web page the string <code>&lt;a
href="goodies.html"&gt;</code> you would use the Tcl command

<blockquote>
<code><pre>
ns_write "&lt;a href=\"goodies.html\"&gt;"
</pre></code>
</blockquote>


So if the majority of your Tcl expressions are simply printing strings,
you'll find it more convenient to avoid the hassles of quoting
by using .adp pages rather than complete .tcl programs.

<p>

Why the use of <code>ns_write</code> instead of <code>puts</code>?
The <code>puts</code> command writes to a program's standard output
stream.  If you were writing a Tcl CGI script, standard output would
be linked through the Web server to the Web client and you could
indeed use <code>puts</code>.  This idea doesn't make sense with
AOLserver, where all the Tcl scripts are running inside the Web server
process.  If the Web server is serving 100 clients simultaneously, to
which client should bytes written with <code>puts</code> be sent?  

<p>

Anyway, if you want to have some fun one night, try redefining
<code>puts</code> to forward its argument to <code>ns_write</code> and
see what happens.

<h3>Lisp Without a Brain</h3>

If in reading this introduction, you've come to realize that "Hey, Tcl
is just like Lisp, but without a brain, and with syntax on steroids",
you might wonder why Lisp isn't a more popular scripting language than
Tcl.  Lisp hasn't been a complete failure, by the way; it is used as
an extension language by users of some popular programs, notably
AutoCAD.  But Tcl has been much more successful.  It has been compiled
into hundreds of larger programs, including AOLserver, which is why we
wrote this book.

<p>

As a software developer, you're unlikely to get rich.  So you might as
well try to get through your life in such a way that you make a
difference to the world.  Tcl illustrates one way:

<ul>

<li>make something that is simple enough for almost everyone to
understand 

<li>give away your source code

<li>explain how to weave your source code in with other systems 

</ul>

In the case of AOLserver, for example, Jim Davidson and Doug McKee had
only a few months to build the whole server from scratch.  They wanted
users to be able to write small programs that ran inside the server
process.  They therefore needed a safe interpreted language so that a
programming error by one user didn't crash the server and bring down all
the Web services for an organization.

<p>

Tcl was available.  Tcl was easy to download and designed to fit
inside larger application programs.  But the Tcl interpreter as
distributed had one terrible bug: it wasn't thread safe, i.e., you
couldn't have two copies of the Tcl interpreter running inside the
same program at the same time.  Doug and Jim had to read through the
Tcl source code and modify it to be thread safe.  So it was critically
important for them that Tcl was open-source <em>and</em> simple enough
so as to not require months or years of study to understand the whole
system.

<p class="stb">

Compare this to Lisp.  Some of the best and brightest
computer scientists raised money to build commercial Lisp
implementations that they then went out and hawked in an indifferent and
confused marketplace.  They succeeded only in breaking their hearts and
their investors' wallets.  A handful of academics produced free
open-source implementations, notably CMU Common Lisp (see <a
href="http://www.cons.org/cmucl/">http://www.cons.org/cmucl/</a>) and
various versions of Scheme (see <a
href="http://www-swiss.ai.mit.edu/scheme-home.html">http://www-swiss.ai.mit.edu/scheme-home.html</a>;
Scheme 48 is the closest to Tcl in spirit).  But these multi-megabyte
monsters weren't designed to fit neatly into someone else's
program.  Nor was there any document explaining how to do it.

<p>

Lisp developers have the satisfaction of knowing that they got it right
30 years before anyone else.  But that's about all they have to show for
40 years of hard work and hundreds of millions of dollars in government
and private funding.  These days, most former Lisp programmers are stuck
using Unix and Microsoft programming environments and, not only do they
have to put up with these inferior environments, but they're saddled
with the mournful knowledge that these environments <em>are</em>
inferior.


<br>
<br>

Continue on to <A HREF="strings.adp"> string operations</A>.



<hr>
Return to <a href="index.adp">Table of Contents</a>
<br>
<br>
<address><a href="mailto:lsandon@alum.mit.edu">lsandon@alum.mit.edu</a></address>

<center><h3>Related Links</h3></center>
	<ul><li><a href="http://brl.sourceforge.net/">BRL - the Beautiful Report Language</a>- Scheme is also available for server-side scripting, with a great SQL interface&nbsp;&nbsp;  <font size=-1>(contributed by <A HREF="/shared/community-member?user_id=177216">Bruce Lewis</a>)</font>
<p>
</ul>
	<center><a href="/comments/add?page_id=4725">Add a comment</a> | <a href="/links/add?page_id=4725">Add a link</a></center>

</body>
</html>
 
